#include "ResourceLoader.h"
GLuint loadTexture(char* path, bool alpha, GLenum target){
	GLuint image;
	SDL_Surface *temp;

	if ( (temp = IMG_Load(path)) ) {
		//Check that the image is square (width equal to height)
		if (temp->w != temp->h) {
			printf("Texture not square");
		}    

		//Check that the image's width is valid and then check that the image's width is a power of 2
		if (temp->w < 1) {
			printf("Invalid texture width");			
		}
		else if ( !((temp->w & (temp->w-1))==0) ) {
			printf("Texture width not power of two\n");
		}    

		//Create the texture
		glGenTextures(1, &image);

		//Load the texture
		glBindTexture(target, image);

		//Generate the texture
		if (alpha)
			glTexImage2D(target, 0, GL_RGBA, temp->w, temp->h, 0, GL_RGBA, GL_UNSIGNED_BYTE, temp->pixels);
		else
			glTexImage2D(target, 0, GL_RGB, temp->w, temp->h, 0, GL_RGB, GL_UNSIGNED_BYTE, temp->pixels);

		//glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
		glTexParameteri(target, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		glTexParameteri(target, GL_TEXTURE_MAG_FILTER, GL_LINEAR);

		SDL_FreeSurface(temp);
		printf("Texture %d loaded.\n", image);
		return image;
	}
	printf("Texture not loaded properly.\n");
	return -1;
	//Shared* shared = Shared::getInstance();
}

char* loadTextFile(char* path){
	FILE *fp;
	char *content = NULL;

	int count=0;

	if (path != NULL) {
		fopen_s(&fp, path,"rt");

		if (fp != NULL) {

			fseek(fp, 0, SEEK_END);
			count = ftell(fp);
			rewind(fp);

			if (count > 0) {
				content = (char *)malloc(sizeof(char) * (count+1));
				count = (int)fread(content,sizeof(char),count,fp);
				content[count] = '\0';
			}
			fclose(fp);
		}
	}
	return content;
}

#define FORMAT_TAG "THINGY_MODEL_v1.05"
StaticObject *loadModel(char *path){
	FILE* file;
	fopen_s(&file, path, "rb");
	if(!file) {
		printf("Unable to open file '%s'\n", path);
		return 0;
	}				          
	// read file version
	char fileversion[sizeof(FORMAT_TAG)+1];
	fread(fileversion, 1, sizeof(FORMAT_TAG), file); 
	if(!strncmp(fileversion, FORMAT_TAG, sizeof(FORMAT_TAG)) == 0){
		printf("File '%s' is not a binary model file.\n", path);
		fclose(file);
		return 0;
	}

	GLuint vc, fc;
	fread(&vc, 1, sizeof(unsigned int), file);
	fread(&fc, 1, sizeof(unsigned int), file);


	GLfloat *vertices = new GLfloat[vc*3];
	GLfloat *normals = new GLfloat[vc*3];
	GLfloat *texcoords = new GLfloat[vc*2];
	GLfloat *colors = new GLfloat[vc*4];
	GLuint *indices = new GLuint[fc*3];
	for(unsigned int i = 0; i<vc*3; i+=3){
		fread(&vertices[i], 1, sizeof(float), file);
		fread(&vertices[i+1], 1, sizeof(float), file);
		fread(&vertices[i+2], 1, sizeof(float), file);
	}
	for(unsigned int i = 0; i<vc*2; i+=2){
		fread(&texcoords[i], 1, sizeof(float), file);
		fread(&texcoords[i+1], 1, sizeof(float), file);
	}
	for(unsigned int i = 0; i<vc*3; i+=3){
		fread(&normals[i], 1, sizeof(float), file);
		fread(&normals[i+1], 1, sizeof(float), file);
		fread(&normals[i+2], 1, sizeof(float), file);
	}
	for(unsigned int i = 0; i<fc*3; i+=3){
		fread(&indices[i], 1, sizeof(unsigned int), file);
		fread(&indices[i+1], 1, sizeof(unsigned int), file);
		fread(&indices[i+2], 1, sizeof(unsigned int), file);
	}
	for(unsigned int i = 0; i<vc*4; i+=4){
		colors[i] = i/(vc*4.0f);
		colors[i+1] = i/(vc*4.0f);
		colors[i+2] = i/(vc*4.0f);
		colors[i+3] = 1.0f;
	}
	fclose(file);
	return new StaticObject(vc, vertices, fc, indices, normals, texcoords, colors);
}
