#include "Scenegraph.h"

Scenegraph::Scenegraph():nodeCount(0), pause(false), visualizeFire(false){
	//shared = Shared::getInstance();
}
Scenegraph::~Scenegraph(){
}

void Scenegraph::addNode(Node *node){
	nodes.push_back(node);
	nodeCount++;
}
int moving = 0;
int strafing = 0;

void Scenegraph::update(){
	handleInput();
	camera->move(Vector((float)strafing,0,(float)moving));
	//calcLOD();
	if(!pause){
		//camera->rotate(AXIS_Z, shared->roll);
		//camera->rotate(AXIS_X, shared->pitch);
		//camera->rotate(AXIS_Y, shared->yaw);
		//camera->move(Vector(0,0, shared->thrust));
		for(uint i = 0; i<nodeCount; i++){
			nodes[i]->update();
		}
	}
}
void Scenegraph::render(){
	if(count){
		frameCount++;
	}
	//glUniform1fARB(loc, time);
	gluPerspective(75, 640 / static_cast<float>(480), 0.1f, 10000.0f);
	glClearColor(0.4f, 0.4f, 0.4f, 1.0f);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	camera->render();

	for(uint i = 0; i<nodeCount; i++){
		if(nodes[i]->type==FIRE){
			if(visualizeFire){
				nodes[i]->render();
			}
		}
		else{
			nodes[i]->render();
		}
	}
	
	SDL_GL_SwapBuffers();
}

void Scenegraph::calcLOD(){
	printf("Calculating scene LOD.\n");
	for(uint i = 0; i<nodeCount; i++){
		nodes[i]->calcLOD();
	}
}

void Scenegraph::setCamera(Camera *camera){
	this->camera = camera;
}

Node::Node(ObjectType type, SceneObject *object)
:type(type), object(object), childCount(0){
	
}
void Node::addChild(Node *node){
	children.push_back(node);
	childCount++;
}
void Node::update(){
	// TODO: updates
	object->update();
	for(uint i = 0; i<childCount; i++){
		children[i]->update();
	}
}
void Node::render(){
	glPushMatrix();
	glPushAttrib( GL_ALL_ATTRIB_BITS ) ;
	// TODO: rendering
	object->render();
	for(uint i = 0; i<childCount; i++){
		children[i]->render();
	}
	glPopAttrib();
	glPopMatrix();
}

void Node::calcLOD(){
	object->calcLOD();
	for(uint i = 0; i<childCount; i++){
		children[i]->calcLOD();
	}
}

void Scenegraph::handleInput(){
	// Check for keyboard input
	SDL_Event event;
	while (SDL_PollEvent(&event)) 
	{
		switch (event.type) 
		{
			case SDL_KEYDOWN:          
				switch (event.key.keysym.sym){
				case 'q':
					stopCount();
					exit(0);
				case 'w':
					moving = -1;
					//shared->pitch = 0;
					break;
				case 's':
					moving = 1;
					//shared->pitch = 0;
					break;
				case 'a':
					strafing = -1;
					//shared->roll = 0;
					break;
				case 'd':
					strafing = 1;
					//camera->move(Vector(1,0,0));
					//shared->roll = 0;
					break;
				case 'z':
					//shared->yaw = 0;
					//camera->rotate(AXIS_Y, 0.1f);
					break;
				case 't':
					startCount();
					break;
				case 'g':
					//shared->thrust = 0;
					break;
				case 'l':
					calcLOD();
					break;
				case 'p':
					pause = !pause;
					break;
				case 'v':
					visualizeFire = !visualizeFire;
					break;
				case 'c':
					camera->printPos();
					break;
				}


				break;
			
			case SDL_KEYUP:
				switch (event.key.keysym.sym){
				case SDLK_ESCAPE:
					exit(0);
				case 'w':
					moving = 0;
					//shared->pitch = 0.001f;
					break;
				case 's':
					moving = 0;
					//shared->pitch = -0.001f;
					break;
				case 'a':
					strafing = 0;
					//shared->roll = -0.001f;
					break;
				case 'd':
					strafing = 0;
					//shared->roll = 0.001f;
					break;
				case 'z':
					//shared->yaw = 0.001f;
					break;
				case 'c':
					//shared->yaw = -0.001f;
					break;
				case 't':
					//shared->thrust = 0.001f;
					break;
				case 'g':
					//shared->thrust = -0.001f;
					break;
				}
				break;
			case SDL_QUIT:
				exit(0);
		}
	}
}
void Scenegraph::startCount(){
	startTime = SDL_GetTicks();
	count = true;
	frameCount = 0;
}
void Scenegraph::stopCount(){
	endTime = SDL_GetTicks();
	float fps = 1000.0 * (float)frameCount/(float)(endTime - startTime);
	printf("Frame rate: %f.\n", fps);
}