#include "TextureTest.h"

Texture2dTest::Texture2dTest(uint width, uint height) : 
						maxWidth(width), maxHeight(height),
						curWidth(width), curHeight(height),
						fbo(new FramebufferObject()),
						active(0){

	texId[0] = loadTexture("textures/test.png", true, GL_TEXTURE_RECTANGLE_ARB);
	glTexParameteri(GL_TEXTURE_RECTANGLE_ARB, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
	glTexParameteri(GL_TEXTURE_RECTANGLE_ARB, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
	glTexParameteri(GL_TEXTURE_RECTANGLE_ARB, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
	glTexParameteri(GL_TEXTURE_RECTANGLE_ARB, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
	texId[1] = loadTexture("textures/test.png", true, GL_TEXTURE_RECTANGLE_ARB);
	glTexParameteri(GL_TEXTURE_RECTANGLE_ARB, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
	glTexParameteri(GL_TEXTURE_RECTANGLE_ARB, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
	glTexParameteri(GL_TEXTURE_RECTANGLE_ARB, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
	glTexParameteri(GL_TEXTURE_RECTANGLE_ARB, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
	
	profile = cgGLGetLatestProfile(CG_GL_FRAGMENT);
	resizeProgram = cgCreateProgramFromFile(cgCreateContext(), 
											CG_SOURCE,
											"cg/lod.cg", 
											profile,
											"resizeTexture", 0);
	renderProgram = cgCreateProgramFromFile(cgCreateContext(), 
											CG_SOURCE,
											"cg/lod.cg", 
											profile,
											"renderTexture", 0);

	if (resizeProgram){
		cgGLLoadProgram(resizeProgram);
	}
	else{
		CGerror cgError = cgGetError();
		if (cgError != CG_NO_ERROR) {
			if (cgError == CG_COMPILER_ERROR) {
				printf("Resize compile error: %s\n", cgGetLastListing(context));
			}
		}
	}
	if (renderProgram){
		cgGLLoadProgram(renderProgram);
	}
	else{
		CGerror cgError = cgGetError();
		if (cgError != CG_NO_ERROR) {
			if (cgError == CG_COMPILER_ERROR) {
				printf("Render compile error: %s\n", cgGetLastListing(context));
			}
		}
	}

	resizeParameter = cgGetNamedParameter(resizeProgram, "oldTexture");
	renderParameter = cgGetNamedParameter(renderProgram, "texture");


}
Texture2dTest::~Texture2dTest(){
}

void Texture2dTest::calcLOD(){
}
void Texture2dTest::calcLod(float lod){
	uint newWidth = (uint)(maxWidth*lod);
	uint newHeight = (uint)(maxHeight*lod);
	float scale = (float)newWidth/curWidth;
	printf("Calculating 2d texture lod: %f\n", lod);
	printf("Scaling 2d texture: %f.\n", scale);
	resize(scale);
	printf("New 2dtexture size: %d / %d.\n", newWidth, maxWidth);
	curWidth = newWidth;
	curHeight = newHeight;
}
void Texture2dTest::render(){
	// Cg program activation
	cgGLBindProgram(renderProgram);
	cgGLEnableProfile(profile);
	cgGLSetTextureParameter(renderParameter, texId[1-active]);
	cgGLEnableTextureParameter(renderParameter);

	// Rendering
	glTranslatef(0, 0, -5.0f);
	float s = (float)curWidth;
	float t = (float)curWidth;
	glBegin(GL_QUADS);
	glTexCoord2f(0,0);
	glVertex3f(0,0,-1.0f);
	glTexCoord2f(s,0);
	glVertex3f(1.0f,0,-1.0f);
	glTexCoord2f(s,t);
	glVertex3f(1.0f,1.0f,-1.0f);
	glTexCoord2f(0,t);
	glVertex3f(0,1.0f,-1.0f);
	glEnd();

	// Cg program deactivation
	cgGLDisableTextureParameter(renderParameter);
	cgGLDisableProfile(profile);
}
void Texture2dTest::resize(float scale){
	// Cg program activation
	cgGLBindProgram(resizeProgram);
	cgGLEnableProfile(profile);
	cgGLSetTextureParameter(resizeParameter, texId[1-active]);
	cgGLEnableTextureParameter(resizeParameter);
	fbo->Bind();
	fbo->AttachTexture(GL_COLOR_ATTACHMENT0_EXT, GL_TEXTURE_RECTANGLE_ARB, texId[active]);
	glDisable(GL_DEPTH_TEST);
	glPushMatrix();
	glLoadIdentity();
	glMatrixMode(GL_PROJECTION);
	glPushMatrix();
	glLoadIdentity();
	glOrtho(0, 1, 0, 1, 0.1, 1000);
	glViewport(0,0,maxWidth, maxHeight);

	// Computation
	float s = (float)maxWidth/scale;
	float t = (float)maxHeight/scale;
	printf("Coordinates used for 2d resizing: %f, %f.\n", s, t);
	glBegin(GL_QUADS);
	glTexCoord2f(0,0);
	glVertex3f(0,0,-1.0f);
	glTexCoord2f(s,0);
	glVertex3f(1.0f,0,-1.0f);
	glTexCoord2f(s,t);
	glVertex3f(1.0f,1.0f,-1.0f);
	glTexCoord2f(0,t);
	glVertex3f(0,1.0f,-1.0f);
	glEnd();

	glPopMatrix();
	glMatrixMode(GL_MODELVIEW);
	glPopMatrix();
	glEnable(GL_DEPTH_TEST);



	// Cg program deactivation
	swap();
	cgGLDisableTextureParameter(resizeParameter);
	cgGLDisableProfile(profile);
	fbo->Disable();
	glViewport(0, 0, WIDTH, HEIGHT);


}
void Texture2dTest::swap(){
	active = 1-active;
	printf("Texture %d active.\n", active);
}
void Texture2dTest::update(){
}

// ******************** Texture3d************************

#include "TextureTest.h"

Texture3dTest::Texture3dTest(uint width, uint height, uint depth) : 
						maxWidth(width), maxHeight(height),
						curWidth(width), curHeight(height),
						curDepth(depth), maxDepth(depth),
						fbo(new FramebufferObject()),
						active(0){

	texId = new uint[2*depth];
	for(uint i = 0; i<depth; i++){
		texId[2*i] = loadTexture("textures/test.png", true, GL_TEXTURE_RECTANGLE_ARB);
		glTexParameteri(GL_TEXTURE_RECTANGLE_ARB, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
		glTexParameteri(GL_TEXTURE_RECTANGLE_ARB, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
		glTexParameteri(GL_TEXTURE_RECTANGLE_ARB, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
		glTexParameteri(GL_TEXTURE_RECTANGLE_ARB, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
		texId[2*i + 1] = loadTexture("textures/test.png", true, GL_TEXTURE_RECTANGLE_ARB);
		glTexParameteri(GL_TEXTURE_RECTANGLE_ARB, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
		glTexParameteri(GL_TEXTURE_RECTANGLE_ARB, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
		glTexParameteri(GL_TEXTURE_RECTANGLE_ARB, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
		glTexParameteri(GL_TEXTURE_RECTANGLE_ARB, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
	}
	
	profile = cgGLGetLatestProfile(CG_GL_FRAGMENT);
	resizeProgram = cgCreateProgramFromFile(cgCreateContext(), 
											CG_SOURCE,
											"cg/lod.cg", 
											profile,
											"resizeTexture", 0);
	renderProgram = cgCreateProgramFromFile(cgCreateContext(), 
											CG_SOURCE,
											"cg/lod.cg", 
											profile,
											"renderTexture", 0);

	if (resizeProgram){
		cgGLLoadProgram(resizeProgram);
	}
	else{
		CGerror cgError = cgGetError();
		if (cgError != CG_NO_ERROR) {
			if (cgError == CG_COMPILER_ERROR) {
				printf("Resize compile error: %s\n", cgGetLastListing(context));
			}
		}
	}
	if (renderProgram){
		cgGLLoadProgram(renderProgram);
	}
	else{
		CGerror cgError = cgGetError();
		if (cgError != CG_NO_ERROR) {
			if (cgError == CG_COMPILER_ERROR) {
				printf("Render compile error: %s\n", cgGetLastListing(context));
			}
		}
	}

	resizeParameter = cgGetNamedParameter(resizeProgram, "oldTexture");
	renderParameter = cgGetNamedParameter(renderProgram, "texture");


}
Texture3dTest::~Texture3dTest(){
}

void Texture3dTest::calcLOD(){
}
void Texture3dTest::calcLod(float lod){
	uint newWidth = (uint)(maxWidth*lod);
	uint newHeight = (uint)(maxHeight*lod);
	uint newDepth = (uint)(maxDepth*lod);
	if(newDepth < 1) newDepth = 1;
	float scale = (float)newWidth/curWidth;
	//printf("Calculating 3d texture lod: %f\n", lod);
	//printf("Scaling 3d texture: %f.\n", scale);
	resize(scale);
	//printf("New 3d texture size: %d / %d.\n", newWidth, maxWidth);
	curWidth = newWidth;
	curHeight = newHeight;
	curDepth = newDepth;
}
void Texture3dTest::render(){
	/*
	// Cg program activation
	cgGLBindProgram(renderProgram);
	cgGLEnableProfile(profile);

	glTranslatef(0, 0, -5.0f);
	for (int i = 0; i<curDepth; i++){

		cgGLSetTextureParameter(renderParameter, texId[1-active + 2*i]);
		cgGLEnableTextureParameter(renderParameter);
	
		// Rendering
		float s = (float)curWidth;
		float t = (float)curWidth;
		glBegin(GL_QUADS);
		glTexCoord2f(0,0);
		glVertex3f(0,0,(float)-i);
		glTexCoord2f(s,0);
		glVertex3f(1.0f,0,(float)-i);
		glTexCoord2f(s,t);
		glVertex3f(1.0f,1.0f,(float)-i);
		glTexCoord2f(0,t);
		glVertex3f(0,1.0f,(float)-i);
		glEnd();
		cgGLDisableTextureParameter(renderParameter);
	}

	// Cg program deactivation
	cgGLDisableProfile(profile);*/
}
void Texture3dTest::resize(float scale){
	// Cg program activation
	cgGLBindProgram(resizeProgram);
	cgGLEnableProfile(profile);
	fbo->Bind();
	glDisable(GL_DEPTH_TEST);
	glPushMatrix();
	glLoadIdentity();
	glMatrixMode(GL_PROJECTION);
	glPushMatrix();
	glLoadIdentity();
	glOrtho(0, 1, 0, 1, 0.1, 1000);
	glViewport(0,0,maxWidth, maxHeight);

	for(int i = 0; i<curDepth*scale; i++){
		int d = (int)(i/scale);
		fbo->AttachTexture(GL_COLOR_ATTACHMENT0_EXT, GL_TEXTURE_RECTANGLE_ARB, texId[active+2*i]);
		cgGLSetTextureParameter(resizeParameter, texId[1-active+2*d]);
		cgGLEnableTextureParameter(resizeParameter);	


		// Computation
		float s = (float)maxWidth/scale;
		float t = (float)maxHeight/scale;
		glBegin(GL_QUADS);
		glTexCoord2f(0,0);
		glVertex3f(0,0,-1.0f);
		glTexCoord2f(s,0);
		glVertex3f(1.0f,0,-1.0f);
		glTexCoord2f(s,t);
		glVertex3f(1.0f,1.0f,-1.0f);
		glTexCoord2f(0,t);
		glVertex3f(0,1.0f,-1.0f);
		glEnd();
		cgGLDisableTextureParameter(resizeParameter);


	}
	glPopMatrix();
	glMatrixMode(GL_MODELVIEW);
	glPopMatrix();
	glEnable(GL_DEPTH_TEST);


	// Cg program deactivation
	swap();
	cgGLDisableProfile(profile);
	fbo->Disable();
	glViewport(0, 0, WIDTH, HEIGHT);






}
void Texture3dTest::swap(){
	active = 1-active;
}
void Texture3dTest::update(){
	calcLod(1.0f);
}