#include "Utilities.h"


/*************************VECTOR CLASS**********************************/
Vector::Vector(float x, float y, float z){
	this->x = x;
	this->y = y;
	this->z = z;
}
Vector::~Vector(){
	
}
Vector Vector::operator+(Vector v){
	return Vector(x + v.x, y + v.y, z + v.z);
}
Vector Vector::operator-(Vector v){
	return Vector(x - v.x, y - v.y, z - v.z);
}
Vector Vector::operator*(Vector v){
	return Vector(y*v.z - z*v.y, z*v.x - x*v.z, x*v.y - y*v.x);
}
Vector Vector::operator*(float d){
	return Vector(x*d, y*d, z*d);
}
Vector Vector::operator/(float d){
	return Vector(x/d, y/d, z/d);
}
void Vector::operator+=(Vector v){
	x += v.x;
	y += v.y;
	z += v.z;
}
void Vector::operator-=(Vector v){
	x -= v.x;
	y -= v.y;
	z -= v.z;
}
void Vector::operator/=(float f){
	x /= f;
	y /= f;
	z /= f;
}
void Vector::operator*=(float f){
	x *= f;
	y *= f;
	z *= f;
}
void Vector::normalize(){
	float l = getLength();
	x /= l;
	y /= l;
	z /= l;
}
void Vector::setZero(){
	x = 0;
	y = 0;
	z = 0;
}
float Vector::dotProduct(Vector v){
	return x*v.x + y*v.y + z*v.z;
}
float Vector::getLength(){
	return sqrt(x*x + y*y + z*z);
}




/***********************QUATERNION CLASS**********************************/
Quaternion::Quaternion(float w, float x, float y, float z){
	this->w = w;
	this->x = x;
	this->y = y;
	this->z = z;
}
Quaternion::~Quaternion(){
	
}
Quaternion Quaternion::operator+(Quaternion q){
	return Quaternion(w + q.w, x + q.x, y + q.y, z + q.z);
}
Quaternion Quaternion::operator-(Quaternion q){
	return Quaternion(w - q.w, x - q.x, y - q.y, z - q.z);
}
Quaternion Quaternion::operator*(Quaternion q){
	return Quaternion(w*q.w - x*q.x - y*q.y - z*q.z, w*q.x + x*q.w + y*q.z - z*q.y, w*q.y - x*q.z + y*q.w + z*q.x, w*q.z + x*q.y - y*q.x + z*q.w);
}
Quaternion Quaternion::operator*(float d){
	return Quaternion(w*d, x*d, y*d, z*d);
}
Quaternion Quaternion::operator/(float d){
	return Quaternion(w/d, x/d, y/d, z/d);
}
void Quaternion::conjunction(Quaternion *q){
	q->w = w;
	q->x = -x;
	q->y = -y;
	q->z = -z;
}
void Quaternion::normalize(){
	float l = sqrt(w*w + x*x + y*y + z*z);
	w /= l;
	x /= l;
	y /= l;
	z /= l;
}
void Quaternion::createMatrix(GLfloat* m){
	m[0] = 1.0f - 2.0f * ( y * y + z * z );
	m[1] = 2.0f * (x * y + z * w);
	m[2] = 2.0f * (x * z - y * w);
	m[3] = 0;
	m[4] = 2.0f * ( x * y - z * w );
	m[5] = 1.0f - 2.0f * ( x * x + z * z );
	m[6] = 2.0f * ( z * y + x * w );
	m[7] = 0;
	m[8] = 2.0f * ( x * z + y * w );
	m[9] = 2.0f * ( y * z - x * w );
	m[10] = 1.0f - 2.0f * ( x * x + y * y );
	m[11] = 0;
	m[12] = 0;
	m[13] = 0;
	m[14] = 0;
	m[15] = 1;
}


/*************************MATRIX CLASS**********************************/
Matrix::Matrix(int width, int height){
	this->width = width;
	this->height = height;
	val = new float[width*height];
	memset(val, 0, sizeof(float)*width*height);
}
Matrix::~Matrix(){
	
}
Matrix Matrix::operator+(Matrix m){
	if(width==m.width && height==m.height){
		Matrix r(width, height);
		for(int i = 0; i<width; i++){
			for(int j = 0; j<height; j++){
				r.val[i+j*width] = val[i+j*width] + m.val[i+j*width];
			}
		}
		return r;
	}
	return Matrix();
}
Matrix Matrix::operator-(Matrix m){
	if(width==m.width && height==m.height){
		Matrix r(width, height);
		for(int i = 0; i<width; i++){
			for(int j = 0; j<height; j++){
				r.val[i+j*width] = val[i+j*width] - m.val[i+j*width];
			}
		}
		return r;
	}
	return Matrix();
}
Matrix Matrix::operator*(Matrix m){
	if(height==m.width){
		Matrix r(width, m.height);
		for(int i = 0; i<width; i++){
			for(int j = 0; j<m.height; j++){
				for(int k = 0; k<height; k++){
					r.val[i+j*width]+=val[k+j*width]*m.val[i+k*width];
				}
			}
		}
		return r;
	}
	return Matrix();
}
Matrix Matrix::operator*(float d){
	Matrix r = clone();
	for(int i = 0; i<width; i++){
		for(int j = 0; j<height; j++){
			r.val[i+j*width] *= d;
		}
	}
	return r;
}
Matrix Matrix::operator/(float d){
	Matrix r = clone();
	for(int i = 0; i<width; i++){
		for(int j = 0; j<height; j++){
			r.val[i+j*width] /= d;
		}
	}
	return r;
}
Matrix Matrix::clone(){
	Matrix r(width, height);
	for(int i = 0; i<width; i++){
		for(int j = 0; j<height; j++){
			r.val[i+j*width] = val[i+j*width];
		}
	}
	return r;
}

/*******************VERTEX CLASS*****************************/
Vertex::Vertex(float x, float y, float z){
	this->x = x;
	this->y = y;
	this->z = z;
}
Vertex::~Vertex(){
	
}

Vertex Vertex::operator+(Vector v){
    return Vertex(x + v.x, y + v.y, z + v.z);
}
Vertex Vertex::operator-(Vector v){
	return Vertex(x - v.x, y - v.y, z - v.z);
}

/*********************EDGE CLASS***********************************/
Edge::Edge(Vertex *v1, Vertex *v2){
	this->v1 = v1;
	this->v2 = v2;
}
Edge::~Edge(){
	
}

Vector Edge::getVector(){
	return Vector((v1->x + v2->x)/2.0f, (v1->y + v2->y)/2.0f, (v1->z + v2->z)/2.0f);
}

/***********************FACE CLASS*******************************/
Face::Face(Vertex *v1, Vertex *v2, Vertex *v3){
	this->v1 = v1;
	this->v2 = v2;
	this->v3 = v3;
	Vector vec1 = Vector(v2->x-v1->x, v2->y-v1->y, v2->z-v1->z);
	Vector vec2 = Vector(v3->x-v1->x, v3->y-v1->y, v3->z-v1->z);
	normal = vec1*vec2;
	normal.normalize();
}
Face::~Face(){
	
}
