#ifndef _UTILITIES_
#define _UTILITIES_
#include "Globals.h"
class Vector;
class Quaternion;
class Matrix;
class Vertex;
class Edge;
class Face;
class Vector{
public:
	float x, y, z;
	Vector(){};
	Vector(float, float, float);
	~Vector();
	Vector operator+(Vector);
	Vector operator-(Vector);
	Vector operator*(Vector);
	Vector operator*(float);
	Vector operator/(float);
	void operator+=(Vector);
	void operator-=(Vector);
	void operator*=(float);
	void operator/=(float);
	void normalize();
	void setZero();
	float dotProduct(Vector);
	float getLength();
};
class Quaternion{
public:
	float w, x, y, z;
	Quaternion(){};
	Quaternion(float, float, float, float);
	~Quaternion();
	Quaternion operator+(Quaternion);
	Quaternion operator-(Quaternion);
	Quaternion operator*(Quaternion);
	Quaternion operator*(float);
	Quaternion operator/(float);
	void normalize();
	void conjunction(Quaternion*);
	void createMatrix(GLfloat*);
};
class Matrix{
public:
	float *val;
	int width, height;
	Matrix(){};
	Matrix(int, int);
	~Matrix();
	Matrix operator+(Matrix);
	Matrix operator-(Matrix);
	Matrix operator*(Matrix);
	Matrix operator*(float);
	Matrix operator/(float);
	Matrix clone();
	Matrix inverse();
};

class Vertex : public Vector{
public:
	Vector normal;
	Face* faces;
	Edge* edges;
	Vertex(){};
	Vertex(float, float, float);
	~Vertex();
	Vertex operator+(Vector);
	Vertex operator-(Vector);

};
class Edge{
private:
public:
	Edge *lNext, *lPrev, *rNext, *rPrev;
	Face *lFace, *rFace;
	Vertex *v1, *v2;
	Edge(){};
	Edge(Vertex*, Vertex*);
	~Edge();
	Vector getVector();
};
class Face{
private:
public:
	Vertex *v1, *v2, *v3;
	Edge *e1, *e2, *e3;
	Vector texCoords[3];
	Vector normal;
	Face(){};
	Face(Vertex*, Vertex*, Vertex*);
	~Face();
};


#endif